package cJTab;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Transparency;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;
import javax.swing.JTabbedPane;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

/**
 * La classe <code>CloseTabbedPaneUI</code> sert  dessiner les onglets
 * @author bcaylux
 *
 */
public class CloseTabbedPaneUI extends BasicTabbedPaneUI {
	    private GeneralPath s = new GeneralPath();
	    private Color selectedColor;

	    
	    public CloseTabbedPaneUI (CTabbedPane pane, Color selectedColor, Color noneSelectedColor) {	        
	        this.selectedColor = selectedColor;
	        this.tabPane = pane;	        
	    }
	    
	    /**
	     * Dessin de l'onglet quand il est en HAUT
	     * @param x l'abcisse de l'onglet
	     * @param y l'ordonne de l'onglet
	     * @param w la largeur de l'onglet
	     * @param h la hauteur de l'onglet
	     * @param selec est-ce un onglet slectionn ?
	     * @return le shape de l'onglet
	     */	    
	    private GeneralPath ongletH(int x, int y, int w, int h, boolean selec){
	    	s.reset();
	    	int b;
	    	if(tabPane.getTabLayoutPolicy()==JTabbedPane.SCROLL_TAB_LAYOUT)b=h-1;
	    	else b = h-3;
	    	if(x+h < x+w-h/2){
	    	  s.append(new CubicCurve2D.Double(x, y+h, x, y-2, x-2, y, x+h, y), true);
			  if(selec)
			        s.append(new CubicCurve2D.Double(x+w-h/2, y, x+w+h/2, y, x+w, y+b, x+w+h, y+b), true);
			  else  s.append(new CubicCurve2D.Double(x+w-h/2, y, x+w, y, x+w, y, x+w, y+h), true);
	    	}
	    	else{
	    		int m = x+w/2;
	    		s.append(new CubicCurve2D.Double(x, y+h, x, y, x, y, m, y), true);
				if(selec)
				      s.append(new CubicCurve2D.Double(x+w-h/2, y, x+w+h/2, y, x+w, y+b, x+w+h, y+b), true);
				else  s.append(new CubicCurve2D.Double(m, y, x+w, y, x+w, y, x+w, y+h), true);

	    	}
	    	return s;
	    }
	   
	    /**
	     * Dessin de l'onglet quand il est en BAS
	     * @param x l'abcisse de l'onglet
	     * @param y l'ordonne de l'onglet
	     * @param w la largeur de l'onglet
	     * @param h la hauteur de l'onglet
	     * @param selec est-ce un onglet slectionn ?
	     * @return le shape de l'onglet
	     */	
	    private GeneralPath ongletB(int x, int y, int w, int h, boolean selec){
	    	s.reset();
	    	int u;
	    	if(tabPane.getTabLayoutPolicy()==JTabbedPane.SCROLL_TAB_LAYOUT)u=1;
	    	else u = 2;
	    	double b = h-u;
	    	if(x+h < x+w-h/2){
	    	  s.append(new CubicCurve2D.Double(x, y, x, y+b, x, y+b, x+h, y+b), true);
			  if(selec)
				     s.append(new CubicCurve2D.Double(x+w-h/2, y+b, x+w+h/2, y+b, x+w, y+u, x+w+h, y+u), true);
			  else   s.append(new CubicCurve2D.Double(x+w-h, y+b, x+w, y+b, x+w, y+b, x+w, y+u), true);
	    	}
	    	else{
	    		int m = x+w/2;
	    		s.append(new CubicCurve2D.Double(x, y, x, y+b, x, y+b, m, y+b), true);
				if(selec)
				      s.append(new CubicCurve2D.Double(x+w-h/2, y+b, x+w+h/2, y+b, x+w, y+u, x+w+h, y+u), true);
				else  s.append(new CubicCurve2D.Double(m, y+b, x+w, y+b, x+w, y+b, x+w, y+u), true);

	    	}
	    	return s;
	    }
	    /**
	     * Dessin de l'onglet quand il est  GAUCHE
	     * @param x l'abcisse de l'onglet
	     * @param y l'ordonne de l'onglet
	     * @param w la largeur de l'onglet
	     * @param h la hauteur de l'onglet
	     * @param selec est-ce un onglet slectionn ?
	     * @return le shape de l'onglet
	     */	
	    
	    private GeneralPath ongletG(int x, int y, int w, int h, boolean selec){
	    	s.reset();
	    	s.append(new Line2D.Double(x+w, y, x+h/2, y), true);
	    	s.append(new CubicCurve2D.Double(x+h/2, y, x, y, x, y, x, y+h/2), true);
	    	s.append(new CubicCurve2D.Double(x, y+h/2, x, y+h, x, y+h, x+h/2, y+h), true);
	    	if(selec)s.append(new CubicCurve2D.Double(x+h/2, y+h, x+w, y+h, x+w, y+h, x+w, y+3*h), true);
	    	else s.append(new Line2D.Double(x+h/2, y+h, x+w, y+h), true);
	    	return s;
	    }
	    
	    /**
	     * Dessin de l'onglet quand il est  DROITE
	     * @param x l'abcisse de l'onglet
	     * @param y l'ordonne de l'onglet
	     * @param w la largeur de l'onglet
	     * @param h la hauteur de l'onglet
	     * @param selec est-ce un onglet slectionn ?
	     * @return le shape de l'onglet
	     */	
	    private GeneralPath ongletD(int x, int y, int w, int h, boolean selec){
	    	s.reset();
	    	--w;
	    	s.append(new Line2D.Double(x, y, x+w-h/2, y), true);
	    	s.append(new CubicCurve2D.Double(x+w-h/2, y, x+w, y, x+w, y, x+w, y+h/2), true);
	    	s.append(new CubicCurve2D.Double(x+w, y+h/2, x+w, y+h, x+w, y+h, x+w-h/2, y+h), true);
	    	if(selec)
	    		 s.append(new CubicCurve2D.Double(x+w-h/2, y+h, x, y+h, x, y+h, x, y+3*h), true);
	    	else s.append(new Line2D.Double(x+w-h/2, y+h, x, y+h), true);
	    	return s;
	    }
	   

	   /**
	    * Cette mthode dessine la bordure de l'onglet.
	    * @param g le graphics
	    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
	    * @param x l'abcisse
	    * @param y l'ordonne
	    * @param w la largeur
	    * @param h la hauteur
	    * @param isSelected <code>true</code> si l'onglet est slectionn, <code>false</code> sinon.
	    */ 
	   protected void paintTabBorder(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected) {
		   Graphics2D g2 =(Graphics2D)g;
	       g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);			 
		   g2.setColor(Color.GRAY);
		   switch (tabPlacement){
		      case JTabbedPane.TOP :    g2.setColor(Color.WHITE);g2.translate(1, 1);
                                        g2.draw(ongletH(x, y, w, h, isSelected));
                                        g2.setColor(Color.GRAY);g2.translate(-1, -1);
                                        g2.draw(ongletH(x, y, w, h, isSelected));
		                                break;
		      case JTabbedPane.BOTTOM : g2.setColor(Color.WHITE);g2.translate(1, 1);
		                                g2.draw(ongletB(x, y, w, h, isSelected));
		                                g2.setColor(Color.GRAY);g2.translate(-1, -1);
		                                g2.draw(ongletB(x, y, w, h, isSelected));
		                                break;
		      case JTabbedPane.LEFT :   g2.setColor(Color.WHITE);g2.translate(1, 1);
		                                g2.draw(ongletG(x, y, w, h, isSelected));
		                                g2.setColor(Color.GRAY);g2.translate(-1, -1);
		                                g2.draw(ongletG(x, y, w, h, isSelected));
		                                break;
		      default :                 g2.setColor(Color.WHITE);g2.translate(0, 1);
		                                g2.draw(ongletD(x, y, w, h, isSelected));
		                                g2.setColor(Color.GRAY);g2.translate(0, -1);
		                                g2.draw(ongletD(x, y, w, h, isSelected));break;
		   }	
	    }
	    
	   /**
	    * Cette mthode peint le fond de l'onglet.
	    * @param g le graphics
	    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
	    * @param x l'abcisse
	    * @param y l'ordonne
	    * @param w la largeur
	    * @param h la hauteur
	    * @param isSelected <code>true</code> si l'onglet est slectionn, <code>false</code> sinon.
	    */ 
	    protected void paintTabBackground(Graphics g, int tabPlacement, int tabIndex, int x, int y, int w, int h, boolean isSelected) {
        	Graphics2D g2 =(Graphics2D)g;
	        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	        if (isSelected) g2.setColor(selectedColor);
	        else            g2.setColor(tabPane.getBackgroundAt(tabIndex));
	        switch (tabPlacement){
		      case JTabbedPane.TOP :    g2.fill(ongletH(x, y, w, h, isSelected));break;
		      case JTabbedPane.BOTTOM : g2.fill(ongletB(x, y, w, h, isSelected));break;
		      case JTabbedPane.LEFT :   g2.fill(ongletG(x, y, w, h, isSelected));break;
		      default :                 g2.fill(ongletD(x, y, w, h, isSelected));break;
		   }	
   
	    }
	    
	    protected void paintContentBorder(Graphics g, int tabPlacement, int selectedIndex) {       
	        // affichage du contenu ...       
	    	super.paintContentBorder(g, tabPlacement, selectedIndex);
	    }
	    

	    protected int getRollOverTab(){
	    	return super.getRolloverTab();
	   	
	    }
		   /**
		    * Cette mthode dessine le trait haut entourant le contenu de l'onglet.
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param selectedIndex l'indice de l'onglet slectionn
		    * @param x l'abcisse
		    * @param y l'ordonne
		    * @param w la largeur
		    * @param h la hauteur
		    */
		protected void paintContentBorderTopEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h) {
		    if(selectedIndex == -1) return; // Correction de B. Caylux
	        
			Rectangle rect = getTabBounds(tabPane, selectedIndex);
            
		    if(tabPlacement==JTabbedPane.TOP){		    
		       g.setColor(this.selectedColor);
		       g.fillRect(x, y, x+w, y+h);	    
		       g.setColor(Color.GRAY);
		       g.drawLine(x , y, rect.x, y);
		       g.drawLine((int)(rect.x+rect.width+rect.height) , y, x+w, y); 
		    }else{
		    	g.setColor(this.selectedColor);
			    g.fillRect(x, y, x+w, y+h);			    
			    g.setColor(Color.GRAY);
			    g.drawLine(x, y, x+w, y); 	
		    }
		}

		   /**
		    * Cette mthode dessine le trait Gauche entourant le contenu de l'onglet.
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param selectedIndex l'indice de l'onglet slectionn
		    * @param x l'abcisse
		    * @param y l'ordonne
		    * @param w la largeur
		    * @param h la hauteur
		    */	
	    protected void paintContentBorderLeftEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h) {
		    if(selectedIndex == -1) return;
		    g.setColor(Color.GRAY);
	    	Rectangle rect = getTabBounds(tabPane, selectedIndex); 
	        if(tabPlacement==JTabbedPane.LEFT){
		    	g.drawLine(x, y, x, y+rect.y);
		    	g.drawLine(x, (int)(rect.y+1.6*rect.height), x, y+h); 

	        }else{ 
		       g.drawLine(x , y, x, y+h);
	        }
	    }
		   /**
		    * Cette mthode dessine le trait droit entourant le contenu de l'onglet.
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param selectedIndex l'indice de l'onglet slectionn
		    * @param x l'abcisse
		    * @param y l'ordonne
		    * @param w la largeur
		    * @param h la hauteur
		    */
		
	    protected void paintContentBorderRightEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h) {

		    if(selectedIndex == -1) return; 
		    g.setColor(Color.GRAY);
	    	Rectangle rect = getTabBounds(tabPane, selectedIndex); 
	        if(tabPlacement==JTabbedPane.RIGHT){
	        	g.drawLine(x+w, y, x+w, y+rect.y);
		    	g.drawLine(x+w, (int)(rect.y+1.6*rect.height), x+w, y+h); 

	        }else{ 
		       g.drawLine(x+w-1, y, x+w-1, y+h);
	        }
		    
	    }
		   /**
		    * Cette mthode dessine le trait en bas entourant le contenu de l'onglet.
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param selectedIndex l'indice de l'onglet slectionn
		    * @param x l'abcisse
		    * @param y l'ordonne
		    * @param w la largeur
		    * @param h la hauteur
		    */
	    protected void paintContentBorderBottomEdge(Graphics g, int tabPlacement, int selectedIndex, int x, int y, int w, int h) {
		    if(selectedIndex == -1) return; 
	        
	    	Rectangle rect = getTabBounds(tabPane, selectedIndex); 
	         g.setColor(Color.GRAY);
	    	if(tabPlacement==JTabbedPane.BOTTOM){
		    	g.drawLine(x , y+h, rect.x, y+h);
		    	g.drawLine((int)(rect.x+rect.width+rect.height) , y+h, x+w, y+h);
	    	}else{
	    	   g.drawLine(x , y+h-1, x+w-1, y+h-1);
	    	}
	    }
	    
	      
		   /**
		    * Cette mthode crit le texte de l'onglet.
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param f la police du texte
		    * @param fm le Font Metrics 
		    * @param tabIndex l'indice de l'onglet
		    * @param titre le titre de l'onglet la largeur
		    * @param textRect le rectangle du texte
		    * @param isSelected l'onglet est-il slectionn ?
		    */

	    protected void paintText(Graphics g, int tabPlacement, Font f, FontMetrics fm,
	            int tabIndex, String titre, Rectangle textRect, boolean isSelected) {
        
	        Graphics2D g2d = (Graphics2D)g;

	        /** Lissage du texte et des dessins */
	        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
	       
	        if (tabPane.getIconAt(tabIndex) instanceof CloseTabIcon)
		        if(tabPane.getSelectedIndex() == tabIndex-1)
		            super.paintText(g2d, tabPlacement, f, fm, tabIndex, titre,textRect, isSelected);
		        else
		            super.paintText(g2d, tabPlacement, f, fm, tabIndex, titre, new Rectangle(textRect.x -7, textRect.y, textRect.width, textRect.height), isSelected);
	        else
		        if(tabPane.getSelectedIndex() == tabIndex-1)
		            super.paintText(g2d, tabPlacement, f, fm, tabIndex, titre, new Rectangle(textRect.x +6, textRect.y, textRect.width, textRect.height), isSelected);
		        else
		            super.paintText(g2d, tabPlacement, f, fm, tabIndex, titre, new Rectangle(textRect.x -1, textRect.y, textRect.width, textRect.height), isSelected);
	    }
	    
		   /**
		    * Cette mthode appele la mthode l'icone. L'icone dans ce cas est compose de 2 <i>morceaux</i> :
		    * <ul>
		    *  <li>L'icone de gauche si elle existe.</li>
		    *  <li>La croix (blanche ou rouge).</li>
		    *  </ul>
		    * @param g le graphics
		    * @param tabPlacement indique si on est haut, bas, droite  ou gauche
		    * @param icon l'icone
		    * @param iconRect le rectangle de l'icone
		    * @param isSelected l'onglet est-il slectionn ?
		    */

	    protected void paintIcon(Graphics g, int tabPlacement, int tabIndex, Icon icon,
	            Rectangle iconRect, boolean isSelected) {
	        if(tabPane.getSelectedIndex() == tabIndex-1)
	            super.paintIcon(g, tabPlacement, tabIndex, icon,  new Rectangle(iconRect.x + 4, iconRect.y, iconRect.width, iconRect.height), isSelected);
	        else
	            super.paintIcon(g, tabPlacement, tabIndex, icon,  new Rectangle(iconRect.x - 3, iconRect.y, iconRect.width, iconRect.height), isSelected);
	   } 
	    
	    
	
	    /**
	     * Retourne la largeur de l'onglet.
	     * @param tabPlacement indique si on est haut, bas, droite  ou gauche
	     * @param tabIndex l'indice de l'onglet
	     * @param metrics le <code>FonMetrics</code> du texte de l'onglet
	     * @return Retourne la largeur de l'onglet.
	     */
	    protected int calculateTabWidth(int tabPlacement, int tabIndex, FontMetrics metrics) {	    
	    	 if (tabPane.getSelectedIndex() == tabIndex-1 )
	    		  // on dcale un peu (de 10) pour que le texte ne soit pas couvert
	    		  // par l'onglet slectionn qui est  sa gauche 
	    		  if((tabPane.getIconAt(tabIndex) instanceof CloseTabIcon))
		               return super.calculateTabWidth(tabPlacement, tabIndex, metrics) + 15;
		          else  return super.calculateTabWidth(tabPlacement, tabIndex, metrics) + 10;
	         else if (tabPane.getIconAt(tabIndex) instanceof CloseTabIcon)
	                   return super.calculateTabWidth(tabPlacement, tabIndex, metrics) + 5;
	              else  return super.calculateTabWidth(tabPlacement, tabIndex, metrics);
	    }

	    
       /**
        * Redfini pour ne pas dessiner le rectangle indicateur de Focus
        */
	    protected void paintFocusIndicator(Graphics g, int tabPlacement,
                Rectangle[] rects, int tabIndex, 
                Rectangle iconRect, Rectangle textRect,
                boolean isSelected) {
	    	//super.paintFocusIndicator(g, tabPlacement, rects, tabIndex, iconRect, textRect, isSelected);
	    	
	    }
	    public Rectangle getTabBounds(int tabIndex, Rectangle dest) {        
	        return super.getTabBounds(tabIndex, dest);
	    }
	    
	}

